/*==========================================================================
//  Implementation of a competitive swarm optimizer (CSO) for large scale optimization
//
//  See the details of CSO in the following paper
//  R. Cheng and Y. Jin, A Competitive Swarm Optimizer for Large Scale Optmization,
//  IEEE Transactions on Cybernetics, 2014
//
//  The test instances are the CEC'08 benchmark functions for large scale optimization
//
//  The source code CSO is implemented by Ran Cheng 
//
//  If you have any questions about the code, please contact: 
//  Ran Cheng at r.cheng@surrey.ac.uk 
//  Prof. Yaochu Jin at yaochu.jin@surrey.ac.uk
===========================================================================*/


#include"stdio.h"
#include"string.h"
#include"rand.h";
#include "func.h"
#include "benchmark.h"


#define MAXPOP 5000
#define MAXDIM 10000

//parameter definition
int popsize,d, maxgen, bestidx;
int randlist[MAXPOP];
double p[MAXPOP][MAXDIM], v[MAXPOP][MAXDIM], fitness[MAXPOP];
double best, bestp[MAXDIM], results[100]; 
double center[MAXDIM];
double phi, seed, upper,lower;
int FEs,funcid, maxFE, maxrun;

//function definition
int input();
int initialize(int funcid);
int getrandlist(int randlist[], int length);
int getcenter();
int evolve(int winidx, int loseidx);
int compete(int r1, int r2);
int boundary(double a[], int dim);
double F(int funcid, double a[], int dim);
int run(int maxrun);
int procedure(int funcid, int run);


int main(){
	input();
	run(maxrun);
	return 0;
}

int run(int maxrun){
	for(funcid = 1; funcid <=7; funcid++){
		for(int run = 1; run <= maxrun; run++)
			procedure(funcid, run);
	}
	return 0;
}


//main procedure
int procedure(int funcid, int runnum){
	int gennum = maxgen;
	initialize(funcid);
	printf("Function %d Run No.%d\n", funcid, runnum);

	while(FEs < maxFE){
		int winidx, loseidx;
		gennum--;
		//generate a random index list in preparation for pairwise competition 
		getrandlist(randlist, popsize);
		//calculate the center (mean position) of the swarm
		getcenter();
		//pairwise competition
		for(int i = 0; i < popsize/2; i++){
			int r1 = randlist[i], r2 = randlist[i + popsize/2];
			winidx = compete(r1, r2);
			loseidx = r1 + r2 - winidx;
			evolve(winidx,loseidx);
		}
		memcpy(bestp, p[bestidx], sizeof(bestp));
	}
	//output result
	printf("Best result: %.6e\n", best);
	return 0;
}


int input(){
	//basic parameter setting
	maxrun = 1;					//max run number
	d = 100;					//problem dimensionality
	maxFE = 5000*d;				//max FEs
	seed = 0.3;					//random seed
	randomize(seed);
	return 0;
}

int initialize(int funcid){


	//boundary setting
	if(funcid == 1 || funcid == 2 || funcid == 3){
		lower = -100; upper = 100;
	}
	if(funcid == 4){
		lower = -5; upper = 5;
	}
	if(funcid == 5){
		lower = -600; upper = 600;
	}
	if(funcid == 6){
		lower = -32; upper = 32;
	}
	if(funcid == 7){
		lower = -1; upper = 1;
	}

	//phi setting (the only parameter in CSO, please SET PROPERLY)
	if(funcid == 1 || funcid == 4 || funcid == 5 || funcid == 6){
		//for seperable functions
		if(d >= 2000)
			phi = 0.2;
		else if(d >= 1000)
			phi = 0.15;
		else if(d >= 500)
			phi = 0.1;
		else
			phi = 0;
	}
	else{
		//for non-seperable functions
		if(d >= 2000)
			phi = 0.2;
		else if(d >= 1000)
			phi = 0.1;
		else if(d >= 500)
			phi = 0.05;
		else
			phi = 0;
	}

	//population size setting
	if(d >= 5000)
		popsize = 1500;
	else if(d >= 2000)
		popsize = 1000;
	else if(d >= 1000)
		popsize = 500;
	else if(d >= 100)
		popsize = 100;


	for(int i = 0; i < popsize; i++)
		for(int j = 0; j < d; j++)
			{
				p[i][j] = rndreal(lower,upper); 
				v[i][j] = 0;
			}
	best = 1e20;


	FEs = 0;
	for(int i = 0; i < popsize; i++){
			fitness[i] = F(funcid, p[i], d);
			if(fitness[i]  < best){
				best = fitness[i] ;
				bestidx = i;
			}
	}
	memcpy(bestp, p[bestidx], sizeof(bestp));
	

	return 0;
}

//loser learn from winner
int evolve(int winidx, int loseidx){
	//position and velocity update
	double c1, c2, c3;
	for(int j = 0; j < d; j++){
		c1 = randomperc();
		c2 = randomperc();
		c3 = randomperc();
		v[loseidx][j] = c1*v[loseidx][j] 
										+ c2*( p[winidx][j] - p[loseidx][j])
										+ c3*phi*(center[j] - p[loseidx][j]);
		p[loseidx][j] += v[loseidx][j];
	}
	//boundary control
	boundary(p[loseidx], d); 
	//fitness evaluation
	fitness[loseidx] = F(funcid, p[loseidx], d);
	if(fitness[loseidx]  < best){
		best = fitness[loseidx] ;
		bestidx = loseidx;
	}
	return 0;
}


//calculate the population center
int getcenter(){
	for(int j = 0; j < d; j++){
		center[j] = 0;
		for(int i = 0; i < popsize; i++){
			center[j] += p[i][j];
		}
		center[j] /= popsize;
	}
	return 0;
}


//generat a random index list ranging from 1 to length
int getrandlist(int randlist[], int length){
	int end,k;
	int templist[MAXPOP];
		end = length - 1;
		for(int i = 0; i < length; i++){
			templist[i] = i;
		}
		for(int i = 0; i < length; i++){
			k = rnd(0,end);
			randlist[i] = templist[k];
			templist[k] = templist[end];
			end--;
		}
	return 0;
}


// compete two selected particles
int compete(int r1, int r2){
	if(fitness[r1] < fitness[r2])
		return r1;
	return r2;
}


int boundary(double a[], int dim){
	for(int i = 0; i < dim; i++){
		if(a[i] < lower)
			a[i] = lower;
		if(a[i] > upper)
			a[i] = upper;
	}
	return 0;
}

double F(int funcid, double a[], int dim){
	FEs++;
	//convergence procedure output, can be commented
	if(FEs % 10000 == 0){
			printf("FEs = %d: %.6e\n", FEs, best);
	}

	//return Sphere(a, dim);
	return func(funcid, dim, a);
}


